import { __awaiter } from "tslib";
import { readFileSync, writeFileSync } from 'fs';
import { join } from 'path';
import { fileURLToPath } from 'url';
const distPath = fileURLToPath(new URL('.', import.meta.url).href);
export default function (options) {
    return {
        name: 'sveltekit-adapter-firebase',
        adapt(builder) {
            return __awaiter(this, void 0, void 0, function* () {
                const { outDir, v2, functionOptions, functionName, nodeVersion, } = Object.assign(Object.assign({ outDir: 'build', v2: true, functionName: 'handler', nodeVersion: '16' }, options), { functionOptions: Object.assign({ concurrency: 500 }, options === null || options === void 0 ? void 0 : options.functionOptions) });
                // empty out existing build directories
                builder.rimraf(outDir);
                builder.log.minor(`Publishing to "${outDir}"`);
                builder.log.minor('Copying assets...');
                const publishDir = `${outDir}${builder.config.kit.paths.base}`;
                builder.writeClient(publishDir);
                builder.writePrerendered(publishDir);
                if (!v2 && functionOptions) {
                    throw new Error('Function options can only be used with v2 functions');
                }
                builder.log.info('Generating cloud function for Firebase...');
                yield generateCloudFunction({
                    builder,
                    outDir,
                    v2,
                    functionName,
                    functionOptions,
                });
                builder.log.info('Generating production package.json for Firebase...');
                generateProductionPackageJson({ outDir, nodeVersion });
            });
        },
    };
}
function generateCloudFunction({ builder, outDir, v2, functionName, functionOptions, }) {
    return __awaiter(this, void 0, void 0, function* () {
        builder.mkdirp(join(outDir, '.firebase', 'functions'));
        builder.writeServer(join(outDir, '.firebase', 'server'));
        const replace = {
            '0SERVER': './server/index.js', // digit prefix prevents CJS build from using this as a variable name, which would also get replaced
        };
        const filter = (name) => {
            if (name === 'lib') {
                return true;
            }
            return !name.includes('adapter') && name.endsWith('.js');
        };
        builder.copy(`${distPath}`, join(outDir, '.firebase'), { filter, replace });
        builder.log.minor('Generating cloud function...');
        const manifest = builder.generateManifest({
            relativePath: '../server',
            format: 'esm',
        });
        const initImport = `import { init } from './../function.js';`;
        const firebaseImportV2 = `import { onRequest } from 'firebase-functions/v2/https';`;
        const functionConstV2 = `export const ${functionName} = onRequest(${functionOptions ? JSON.stringify(functionOptions) + ', ' : ''}init(${manifest}));`;
        const firebaseImportV1 = `import { https } from 'firebase-functions';`;
        const functionConstV1 = `export const ${functionName} = https.onRequest(init(${manifest}));`;
        const renderFunctionFile = `${initImport}\n${v2 ? firebaseImportV2 : firebaseImportV1}\n\n${v2 ? functionConstV2 : functionConstV1}\n`;
        writeFileSync(join(outDir, '.firebase', 'functions', 'render.js'), renderFunctionFile);
    });
}
function generateProductionPackageJson({ outDir, nodeVersion, }) {
    const packageJsonString = readFileSync('package.json');
    const packageJson = JSON.parse(packageJsonString.toString());
    const firebaseConfig = {
        dependencies: Object.assign(Object.assign({}, packageJson === null || packageJson === void 0 ? void 0 : packageJson.dependencies), { 'firebase-functions': '^4.0.1' }),
        main: '.firebase/functions/render.js',
        engines: {
            node: nodeVersion,
        },
        type: 'module',
    };
    const updatedPackageJson = Object.assign(Object.assign({}, packageJson), firebaseConfig);
    //Firebase doesn't handle overlapping dev and prod dependencies well, so we delete devDeps as they are not needed anyway
    delete updatedPackageJson.devDependencies;
    writeFileSync(join(outDir, 'package.json'), JSON.stringify(updatedPackageJson, null, 2));
}
